/*******************************************************************************
**  PROJECT NAME : OpenGL ES (GLUT32) Voids Bullets !
**
**  Concept :
**          You are a flight pilot.
**          Oneday , you are in test piloting with your flight.
** 
** 
**  (C)Copyright2006 , Min Chuk , Kim ( rageworx@gmail.com )
**
**  All Program codes protects by copyright-law of international. 
********************************************************************************/
#ifdef _WIN32
	#include <windows.h>
	#include <stdio.h>
	#include <math.h>
	#include <gl/glut.h>
	#include "sounde.h"
#else
	#include <stdio.h>
	#include <math.h>
	#include "glwrap.h"
#endif
#include "app.h"
#include "strings.h"

/*********************** Definations for game *********************************/

#define LEFT_LIMIT 				-2.2
#define RIGHT_LIMIT 			2.2
#define TOP_LIMIT				-2.2
#define BOTTOM_LIMIT			2.2

#define BULLET_SPEED 			0.025
#define DEF_FPS_LIMIT			1000/60
#define DEF_DEPTH_0				-0.009
#define DEF_DEPTH_1				-0.008
#define DEF_DEPTH_2				-0.007
#define DEF_DEPTH_3				-0.006
#define DEF_DEPTH_4				-0.005
#define DEF_DEPTH_5				-0.004

#define DEF_FILE_FLIGHT			"TEX/texture_flight.tex"
#define DEF_FILE_FONT			"TEX/texture_fonts.tex"
#define DEF_FILE_ITEMS			"TEX/texture_items.tex"
#define DEF_FILE_SAPCE			"TEX/texture_space.tex"
#define DEF_FILE_TEXTS			"TEX/texture_texts.tex"
#define DEF_FILE_TITLE			"TEX/texture_title.tex"
#define DEF_FILE_TSCREEN		"TEX/texture_tscreen.tex"
#define DEF_FILE_HOW2PLAY		"TEX/texture_how2play.tex"

#define DEF_FILE_MUSIC1			"SND/op_music.wav"
#define DEF_FILE_MUSIC2			"SND/playmusc.wav"
#define DEF_FILE_MUSIC3			"SND/deadmusc.wav"
#define DEF_FILE_MBUTTON		"SND/button.wav"
#define DEF_FILE_MBLOW			"SND/Blow.wav"
#define DEF_FILE_MMOVE			"SND/mcn_move.wav"
#define DEF_FILE_MSTART			"SND/start.wav"
#define DEF_FILE_MWARNING		"SND/warning.wav"

#define DEF_MAX_BLOW			3

#ifdef _WIN32
	#define DEF_MAX_BULLETS				100
	#define DEF_MAX_CREATE_BULLETS		20
	#define DEF_MAX_RECREATE_BULLETS	30
	#define DEF_CREATE_INTERVAL			800
#else
	#define DEF_MAX_BULLETS				50
	#define DEF_MAX_CREATE_BULLETS		10
	#define DEF_MAX_RECREATE_BULLETS	20
	#define DEF_CREATE_INTERVAL			500
#endif

/*********************** Structure and enums **********************************/
typedef enum{
	DA_TEX_LOADING	= 0,
	DA_TEX_FLIGHT,
	DA_TEX_FONT,
	DA_TEX_ITEMS,
	DA_TEX_SPACE,
	DA_TEX_TEXTS,
	DA_TEX_TITLE,
	DA_TEX_TSCREEN,
	DA_TEX_HOW2PLAY,
	DA_TEX_MAX
}TypeTextures;

typedef enum{
	GSTATE_NONE		= 0,
	GSTATE_TITLE,
	GSTATE_LEVEL,
	GSTATE_OPTION,
	GSTATE_HOW2PLAY,
	GSTATE_GAME_START,
	GSTATE_GAME_PLAYING,
	GSTATE_GAME_DIED,
	GSTATE_GAME_END
}TypeGameState;

typedef struct _STplayer{
	char	moveUP;
	char	moveDOWN;
	char	moveLEFT;
	char	moveRIGHT;
	
	GLfloat	coord_X;
	GLfloat coord_Y;
	
	GLfloat angle_X;
	GLfloat angle_Y;
}STplayer;

typedef struct _STbullets{
	char	onScreen;
	
	char	moveLEFT;
	char	moveRIGHT;
	
	GLfloat speed;
	GLfloat coordX;
	GLfloat coordY;
}STbullet;

/*********************** Externing datas **************************************/

extern GLfloat Craft_vtx[][3];
extern GLubyte Craft_idx[];
extern GLfloat Craft_tex[][2];
extern GLfloat CraftNum_norm[6][3];
extern GLfloat CraftNum_arrays[6][3];
extern GLfloat crushroot[8][2];

extern unsigned char texture_loading[];

extern GLfloat plane_vertex[];
extern GLfloat plane_normal[];
extern GLfloat plane_texcoord[];
extern GLfloat half_plane_vertex[];
extern GLfloat half_up_plane_texcoord[];
extern GLfloat half_down_plane_texcoord[];
extern GLfloat menu_selectbar_texcoord[];
extern GLfloat menu_items_coords[];
extern GLfloat menu_items_coordinate[];
extern GLfloat panel_vertex[];
extern GLfloat panel_texcoord[];
extern GLfloat panel_mask_texcoord[];
extern GLfloat gameover_texcoord[];
extern GLfloat gameover_mask_texcoord[];
extern GLfloat Score_arrays[];
extern GLfloat bullet_vertex[];
extern GLfloat green_bullet_texcoord[];
extern GLfloat red_bullet_texcoord[];

/********************** Const Variables ***************************************/

const 	GLfloat 	light_pos[] 		= {-5, 5, 5, 1};
const 	GLfloat 	light_col[] 		= {1.0, 1.0, 1.0, 1.0};
const 	GLfloat 	light_amb[] 		= {0.7, 0.7, 0.7, 1.0};
const 	GLfloat 	light_spc[] 		= {1.0, 1.0, 1.0, 1.0};
const 	GLfloat 	materialCol[]     	= { 1 , 1 , 1 , 1 };

/********************** Static Variables **************************************/

static	char		ShowedLoading	= 0;
static	char		LoadFromFS 		= 0;

static	int			GameState		= GSTATE_NONE;
static	int			MusicState		= GSTATE_NONE;

static	int			crushed			= 0;
static 	int 		crushcount 		= 0;

static 	GLfloat 	crushcoord[20][2];
static 	GLfloat 	Craft_arrays[60][3];

static	STplayer	player;
static	STbullet	bullets[DEF_MAX_BULLETS];
static	unsigned	start_time		= 0;
static	unsigned	passed_time		= 0;
static	int 		totscore 		= 0;
static	int			create_interval = 0;
static	int			hit_level		= 0;
static  unsigned	game_score		= 0;

/********************** Local Variables ***************************************/

GLfloat 	eye_sep 				= 4.0f;
GLfloat 	focal_len 				= 5.0f;

GLuint  	TextureID[DA_TEX_MAX] 	= {0};
void    	*Textures[DA_TEX_MAX] 	= {NULL};

GLfloat		menu_items_texcoords[11 * 6 * 2] 	= {0.0f};
GLfloat		menu_items_vertex[11 * 6 * 3]		= {0.0f};

int			menu_item_select		= 0;
char 		ScoreNum[12] 			= { 0 };

void InitGL(void);
void UpdateGL(void);
void DrawGL(void);
void CloseGL(void);

/********************************************************************
** Write a score to file system
*********************************************************************/
void WriteScore(unsigned int socre)
{
#ifdef _WIN32
	FILE *fp;
	
	fp = fopen("SCORE.TXT","w");
	
	if (!fp) return;
	
	fprintf(fp,"The Voids Bullets !!!\n");
	fprintf(fp,"=====================\n");
	fprintf(fp,"(C)2005,2006 rage freeware , Min Chul Kim (denkami@gmail.com)\n");
	fprintf(fp,"\n");
	fprintf(fp,"You have %d scores !! \n",socre);
	
	fclose(fp);
#endif
}

/********************************************************************
** Set perspective ratio on screen.
** it have been exists in GLUT32 as glPersfective(), 
** but it is not in OpenGLES. please do not use glPersfective() !!!
*********************************************************************/
void setPersfective(GLfloat fovy,GLfloat aspect,GLfloat zNear,GLfloat zFar)
{
    GLfloat xmin, xmax, ymin, ymax;
    
    ymax = zNear * (GLfloat)tan(fovy * 3.14 / 360);
    ymin = -ymax;
    xmin = ymin * aspect;
    xmax = ymax * aspect;
    
    glFrustum(xmin,xmax,ymin,ymax,zNear,zFar);
}

/********************************************************************
** Get file size from file system
** Must be changed code on difference platform
*********************************************************************/
unsigned long get_fSize(char *fn)
{
#ifdef _WIN32
    DWORD    FileSizeHigh, FileSizeLow; 
    HANDLE   hFindFile; 
    OFSTRUCT ReOpenBuff; 
     
    hFindFile   = (HANDLE)OpenFile((char*)fn, &ReOpenBuff, OF_READ); 
    FileSizeLow = GetFileSize(hFindFile, &FileSizeHigh); 
    
    CloseHandle(hFindFile);
    
    return FileSizeLow;
#else
    return 0;
#endif
}    

/********************************************************************
** Load texture from file system.
*********************************************************************/
void* LoadTextureFromFS(char* TextureFile)
{
#ifdef _WIN32    
    static void*    szBuffer = NULL;
    unsigned        filesize = 0;
    
    FILE *fp = fopen(TextureFile,"rb");
    
    if (fp)
    {
        filesize = get_fSize(TextureFile);
        
        if (filesize == 0) return (void*)NULL;
        
        if (szBuffer) free(szBuffer);
        szBuffer = (void*)malloc(filesize + 1);
        
        memset(szBuffer,0,filesize);
        
        fread(szBuffer,1,filesize,fp);
        
        fclose(fp);
        
        return (void*)szBuffer;
    }
    else
    
    return (void*)NULL;
#else
    return (void*)NULL;
#endif
}

/********************************************************************
** Binding textures to memory.
*********************************************************************/
void BindingTexture(void* pTexture,int ID,int szWidth,int szHeight)
{
    int aafilter = GL_LINEAR;
    
    if (pTexture)
    {
        Textures[ID] = pTexture;
        
        glBindTexture(GL_TEXTURE_2D,TextureID[ID]);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, szWidth,szHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,Textures[ID]);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, aafilter);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, aafilter);
    }
    else
    {
        if (Textures[ID]) free(Textures[ID]);
        Textures[ID] = malloc(szWidth * szHeight * 3 +1);
        memset(Textures[ID],0,szWidth * szHeight * 3 +1);

        glBindTexture(GL_TEXTURE_2D,TextureID[ID]);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, szWidth,szHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,Textures[ID]);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, aafilter);
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, aafilter);    
    }
    
}

/********************************************************************
** Makes coordinate of menu textures.
*********************************************************************/
void MakeMenuCoordinates(void)
{
	int 	ref_index 	= 0;
	int 	index 		= 0;
	GLfloat	calb		= 0.0f;
	
	// First , Makes texture coordinates 	
	for (ref_index = 0;ref_index < 11;ref_index++)
	{
		index = ref_index * (6 * 2);
		
		// 00 - x , y
		menu_items_texcoords[index +  0] = 1.0f;
		menu_items_texcoords[index +  1] = menu_items_coordinate[ref_index * 2 + 0];
		
		// 01 - x , y
		menu_items_texcoords[index +  2] = 0.0f;
		menu_items_texcoords[index +  3] = menu_items_coordinate[ref_index * 2 + 0];
		
		// 02 - x , y
		menu_items_texcoords[index +  4] = 0.0f;
		menu_items_texcoords[index +  5] = menu_items_coordinate[ref_index * 2 + 1];
		
		// 03 - x , y
		menu_items_texcoords[index +  6] = 0.0f;
		menu_items_texcoords[index +  7] = menu_items_coordinate[ref_index * 2 + 1];
		
		// 04 - x , y
		menu_items_texcoords[index +  8] = 1.0f;
		menu_items_texcoords[index +  9] = menu_items_coordinate[ref_index * 2 + 1];
		
		// 05 - x , y
		menu_items_texcoords[index + 10] = 1.0f;
		menu_items_texcoords[index + 11] = menu_items_coordinate[ref_index * 2 + 0];
	}
	
	// Second , Makes vertice coordinates.
	for (ref_index = 0;ref_index < 11;ref_index++)
	{
		index = ref_index * (6 * 3);
		
		switch(ref_index)
		{
			case 0 : // START A NEW GAME
			case 1 : // CHALLENGE A LEVEL
			case 2 : // OPTIONS
			case 3 : // VIEW HOW TO PLAY ?
			case 4 : // EXIT
				calb = 0.0f;
				break;
			case 5 : //  -- SELECT LEVEL --
			case 6 : // [LOW][MED][HIGH][DEAD]
				calb = menu_items_coordinate[5 * 2 + 1] * -1.0f;
			case 7 : // -- OPTIONS --
			case 8 : // SOUND : ON OFF
			case 9 : // VIBRATION : ON OFF
				calb = menu_items_coordinate[8 * 2 + 1] * -1.0f;
			case 10:
				calb = menu_items_coordinate[9 * 2 + 1] * -1.0f;
		}
		
		// 00 - x , y
		menu_items_vertex[index +  0] = 1.0f;
		menu_items_vertex[index +  1] = menu_items_coordinate[ref_index * 2 + 0] * 2 + calb;
		menu_items_vertex[index +  2] = 1.0f;
		
		// 01 - x , y
		menu_items_vertex[index +  3] = -1.0f;
		menu_items_vertex[index +  4] = menu_items_coordinate[ref_index * 2 + 0] * 2 + calb;
		menu_items_vertex[index +  5] = 1.0f;
		
		// 02 - x , y
		menu_items_vertex[index +  6] = -1.0f;
		menu_items_vertex[index +  7] = menu_items_coordinate[ref_index * 2 + 1] * 2 + calb;
		menu_items_vertex[index +  8] = -1.0f;
		
		// 03 - x , y
		menu_items_vertex[index +  9] = -1.0f;
		menu_items_vertex[index + 10] = menu_items_coordinate[ref_index * 2 + 1] * 2 + calb;
		menu_items_vertex[index + 11] = -1.0f;
		
		// 04 - x , y
		menu_items_vertex[index + 12] = 1.0f;
		menu_items_vertex[index + 13] = menu_items_coordinate[ref_index * 2 + 1] * 2 + calb;
		menu_items_vertex[index + 14] = -1.0f;
		
		// 05 - x , y
		menu_items_vertex[index + 15] = 1.0f;
		menu_items_vertex[index + 16] = menu_items_coordinate[ref_index * 2 + 0] * 2 + calb;
		menu_items_vertex[index + 17] = 1.0f;
	}
}

/********************************************************************
** Make the objects 
*********************************************************************/
void SetArrays(void)
{
	int count = 0;
	int num;
	for(count=0;count<60;count++)
	{
		num=Craft_idx[count];
		Craft_arrays[count][0]=Craft_vtx[num][0];
		Craft_arrays[count][1]=Craft_vtx[num][1];
		Craft_arrays[count][2]=Craft_vtx[num][2];
	}
}

/********************************************************************
** Draw the Space World
*********************************************************************/
void DrawSpace(void)
{
    glDisable(GL_DEPTH_TEST);

    glPushMatrix();

    glDisable(GL_BLEND);
    
    glTranslatef(0,0,DEF_DEPTH_0);
    glScalef(1.0f,1.0f,1.0f);

    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_SPACE]);
    glTexCoordPointer(2,GL_FLOAT,0,plane_texcoord);
    glNormalPointer(GL_FLOAT,0,plane_normal);
    glVertexPointer(3,GL_FLOAT,0,plane_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glPopMatrix();    

    glEnable(GL_DEPTH_TEST);
}

/********************************************************************
** Draw the Aircraft of ours
** It contains crush algorithm.
*********************************************************************/
void DrawCraft(void)
{
	int count;//, rnd;
	int count2=0;
	
	GLfloat DrawScaleX = 0.35f;
	GLfloat DrawScaleY = 0.35f;
	GLfloat DrawScaleZ = 0.35f;
	
	if(!crushed)
	{	
		glPushMatrix(); 
		glTranslatef(player.coord_X,player.coord_Y,DEF_DEPTH_3);
		glRotatef(-180,1,0,0);
		glRotatef(-180,0,0,1);
		glScalef(DrawScaleX,DrawScaleY,DrawScaleZ);
		glRotatef(-player.angle_X,0,1,0);
		glRotatef(-player.angle_Y,1,0,0);
		glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_FLIGHT]);
		glVertexPointer(3,GL_FLOAT, 0, Craft_arrays);
		glNormalPointer(GL_FLOAT, 0, CraftNum_norm);
		glTexCoordPointer(2, GL_FLOAT, 0, Craft_tex);
		glDrawArrays(GL_TRIANGLES,0,60);
		glPopMatrix();	

		// Draws Hits ... Burnning ...
		if (hit_level)
		{
		    glDisable(GL_DEPTH_TEST);
		
		    glPushMatrix();
		
		    glDisable(GL_DEPTH_TEST);
		    glDisableClientState(GL_NORMAL_ARRAY);
		    glDisable(GL_LIGHTING);
			    
		    glTranslatef(player.coord_X,player.coord_Y,DEF_DEPTH_4);
		    glScalef(3.0f,5.0f,0.0f);
		
		    glBlendFunc(GL_DST_COLOR, GL_ONE);
		    glEnable(GL_BLEND);
		
		    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_ITEMS]);
		    
			glTexCoordPointer(2,GL_FLOAT,0,red_bullet_texcoord);
		    glVertexPointer(3,GL_FLOAT,0,bullet_vertex);

			for(count=0;count<hit_level;count++)
			{
		    	glDrawArrays(GL_TRIANGLES,0,6);
			}
		
		    glPopMatrix();    
		
			glDisable(GL_BLEND);		
		    glEnable(GL_DEPTH_TEST);
		    glEnable(GL_LIGHTING);
		    glEnableClientState(GL_NORMAL_ARRAY);
		}
	}else
	{
		if(crushcount<1)
		{
			for(count=0;count<20;count++)
			{
				crushcoord[count][0]=crushroot[count2][0];
				crushcoord[count][1]=crushroot[count2][1];
				if(count2<7)count2++; else if(count2==7) count2=0;
			}
		}
		for(count=0;count<20;count++)
		{
			glPushMatrix(); 
			glTranslatef(player.coord_X,player.coord_Y,DEF_DEPTH_3);
			glRotatef(-180,1,0,0);
			glRotatef(-180,0,0,1);
			glScalef(DrawScaleX,DrawScaleY,DrawScaleZ);
			glVertexPointer(3,GL_FLOAT, 0, Craft_arrays);
			glNormalPointer(GL_FLOAT, 0, CraftNum_norm);
			glTexCoordPointer(2, GL_FLOAT, 0, Craft_tex);
			glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_FLIGHT]);
			glTranslatef(crushcoord[count][0],crushcoord[count][1],0);
			glDrawArrays(GL_TRIANGLES,count*3,3);
			crushcoord[count][0]=crushcoord[count][0]*1.2;
			crushcoord[count][1]=crushcoord[count][1]*1.2;
			glPopMatrix();	
		}	
		crushcount++;
	}
}

/********************************************************************
** Reset position one bullet.
*********************************************************************/
void ResetBullets(char number)
{
	char moveWhere = 0;
	
	memset(&bullets[number],0,sizeof(STbullet));
	
	bullets[number].coordX = rand()%640 / 320.0f - 0.8f;
	bullets[number].coordY = 1.0f;
	bullets[number].speed  = 0.01 + ((rand()%10 - 5.f) / 1000.0f);

	moveWhere = rand()%3;

	if (moveWhere == 1)	bullets[number].moveLEFT = 1;
	else
	if (moveWhere == 2) bullets[number].moveRIGHT = 1;
	
	bullets[number].onScreen = 1;
	
	Sleep(1);
}

/********************************************************************
** Reset all bullets.
*********************************************************************/
void ResetAllBullets(void)
{
	unsigned char count;
	
	for(count=0;count<DEF_MAX_BULLETS;count++)
	{
		memset(&bullets[count],0,sizeof(STbullet));
	}
}

/********************************************************************
** Set craft to destroyed.
*********************************************************************/
void DoCrush(void)
{	
	WriteScore(game_score);
	StopPlay(3);
	PlayFile(3,"SND/Blow.wav",0);
	hit_level++;
	
	if ((hit_level>DEF_MAX_BLOW) && (GameState != GSTATE_GAME_DIED))
	{
		crushed = 1;
		GameState = GSTATE_GAME_DIED;
	}
	else
	{
		crushed = 0;
	}
}

/********************************************************************
** Calculate moving algorithm of bullets
*********************************************************************/
void MoveBullets(void)
{
	unsigned char count;
	char onscreen_bullets = 0;
	char creates;
	static
	int hit_interval;
	
	GLfloat bulletX	  = 0.0f;
	GLfloat bulletY   = 0.0f;
	GLfloat checkXmin = 0.0f;
	GLfloat checkXmax = 0.0f;
	GLfloat checkYmin = 0.0f;
	GLfloat checkYmax = 0.0f;
	
	if (crushed) return;
	
	if (hit_level) hit_interval++;
	
	for(count=0;count<DEF_MAX_BULLETS;count++)
	{
		if (bullets[count].onScreen)
		{
			onscreen_bullets ++;
		}
		
		bullets[count].coordY -= bullets[count].speed;
		
		if (bullets[count].moveLEFT) bullets[count].coordX -= bullets[count].speed / 2;
		else
		if (bullets[count].moveRIGHT) bullets[count].coordX += bullets[count].speed / 2;
		
		// check bullet is on screen.
		
		if ((bullets[count].coordX > 1.0f) || (bullets[count].coordX < -1.0f)) 
		{
			bullets[count].onScreen = 0;
			onscreen_bullets--;
		}
		
		if (bullets[count].coordY < -1.0f)
		{
			if (creates < DEF_MAX_RECREATE_BULLETS)
			{
				ResetBullets(count);
				onscreen_bullets++;
				creates++;
			} else 
			{
				bullets[count].onScreen = 0;
				onscreen_bullets--;
				creates--;
			}
		}
		
		// Conflict check;
		bulletX   = bullets[count].coordX;
		bulletY   = bullets[count].coordY;
		
		checkXmin = (player.coord_X - 0.05f);
		checkXmax = (player.coord_X + 0.05f);
		checkYmin = (player.coord_Y - 0.03f);
		checkYmax = (player.coord_Y + 0.03f);

		if ((!crushed) && (bullets[count].onScreen))
		{
			if ((bulletX > checkXmin) && (bulletX < checkXmax))
			{
				if ((bulletY > checkYmin) && (bulletY < checkYmax))
				{
					bullets[count].onScreen = 0;
					onscreen_bullets--;
					creates--;
					
					if (!hit_level)
					{
						DoCrush();
					}
					else
					{
						if(hit_interval>50)
						{
							hit_interval = 0;
							DoCrush();
						}
						else
						{
							hit_interval++;
						}
					}
					
				}
			}
		}

	}
	
	// Makes many bullets if it have time !
	if (onscreen_bullets < DEF_MAX_BULLETS)
	{
		if (create_interval > DEF_CREATE_INTERVAL)
		{
			for (count=0;count<DEF_MAX_BULLETS;count++)
			{
				if ((!bullets[count].onScreen) && (creates < DEF_MAX_RECREATE_BULLETS))
				{
					ResetBullets(count);
					
					creates++;
				}
			}
			create_interval = 0;
			StopPlay(2);
			PlayFile(2,DEF_FILE_MWARNING,0);
		}
		else create_interval++;
	}
}

/********************************************************************
** Draws a blowing effect.
*********************************************************************/
void DrawBlow(void)
{
	static
	GLfloat enlarge = 1.0f;
	
	if (!crushed)
	{
		enlarge = 1.0f;
		return;
	}
	
	if (enlarge >= 10.0f)
	{
		return;
	}
	
    glDisable(GL_DEPTH_TEST);

    glPushMatrix();

    glDisable(GL_DEPTH_TEST);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisable(GL_LIGHTING);
	    
    glTranslatef(player.coord_X,player.coord_Y,DEF_DEPTH_4);
    glScalef(enlarge,enlarge,0.0f);

    glBlendFunc(GL_ONE, GL_ONE);
    glEnable(GL_BLEND);

    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_ITEMS]);
    
	glTexCoordPointer(2,GL_FLOAT,0,red_bullet_texcoord);
    glVertexPointer(3,GL_FLOAT,0,bullet_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glPopMatrix();    

	glDisable(GL_BLEND);		
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnableClientState(GL_NORMAL_ARRAY);
    
    enlarge += 0.5f;
}

/********************************************************************
** Draws bullets.
*********************************************************************/
void DrawBullets(void)
{
	unsigned char count;
	
	for (count=0;count<DEF_MAX_BULLETS;count++)
	{
		if (bullets[count].onScreen)
		{
		    glDisable(GL_DEPTH_TEST);
		
		    glPushMatrix();
		
		    glDisable(GL_DEPTH_TEST);
		    glDisableClientState(GL_NORMAL_ARRAY);
		    glDisable(GL_LIGHTING);
			    
		    glTranslatef(bullets[count].coordX,bullets[count].coordY,DEF_DEPTH_4);
		
		    glBlendFunc(GL_ONE, GL_ONE);
		    glEnable(GL_BLEND);

		    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_ITEMS]);
		    
		    if (bullets[count].speed >= 0.008f)
		    {
		    	glTexCoordPointer(2,GL_FLOAT,0,red_bullet_texcoord);
			}
			else
			{
		    	glTexCoordPointer(2,GL_FLOAT,0,green_bullet_texcoord);
			}
		    glVertexPointer(3,GL_FLOAT,0,bullet_vertex);
		    glDrawArrays(GL_TRIANGLES,0,6);
		
		    glPopMatrix();    

			glDisable(GL_BLEND);		
		    glEnable(GL_DEPTH_TEST);
		    glEnable(GL_LIGHTING);
		    glEnableClientState(GL_NORMAL_ARRAY);
		}
	}
	
}

/********************************************************************
** Calculate the score. 
*********************************************************************/
void CalScore(void)
{
	char score_str[12] = {0};
	int cnt;
	char max;
	
	game_score += 10;
	game_score += passed_time / 1000;
	
	sprintf(score_str,"%12d",game_score);

	for(cnt=0;cnt<12;cnt++)
	{
		if ((score_str[cnt]>='0')&&(score_str[cnt]<='9'))
		{
			ScoreNum[cnt] = score_str[cnt] - '0';
		}
		else
		{
			ScoreNum[cnt] = 0;
		}
	}
	
}

/********************************************************************
** Draws GAME-OVER.
*********************************************************************/
void DrawGameOver(void)
{
    glDisable(GL_DEPTH_TEST);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisable(GL_LIGHTING);

	glPushMatrix();
			
    glTranslatef(0,0,DEF_DEPTH_5);
    glScalef(0.5,0.3,0.5);

    glBlendFunc(GL_DST_COLOR, GL_ZERO);
    glEnable(GL_BLEND);
    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_ITEMS]);
    glTexCoordPointer(2,GL_FLOAT,0,gameover_mask_texcoord);
    glVertexPointer(3,GL_FLOAT,0,plane_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glBlendFunc(GL_ONE, GL_ONE);
    glEnable(GL_BLEND);
    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_ITEMS]);
    glTexCoordPointer(2,GL_FLOAT,0,gameover_texcoord);
    glVertexPointer(3,GL_FLOAT,0,plane_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glPopMatrix();
    
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnableClientState(GL_NORMAL_ARRAY);
}

/********************************************************************
** Draws Score Panel.
*********************************************************************/
void DrawScorePanel(void)
{
	static
	GLfloat	Downing =  0.2f;
	
	if (!crushed)
	{
		Downing -= 0.02f;
	}
	else
	{
		if(crushcount > 99)
		{
			Downing += 0.02f;
		}
	}
	
	if (Downing < 0.0f) Downing = 0.0f;
	if (Downing > 0.2f) Downing = 0.2f;
	
    glDisable(GL_DEPTH_TEST);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisable(GL_LIGHTING);

	glPushMatrix();
			
    glTranslatef(0,Downing,DEF_DEPTH_4);

    glBlendFunc(GL_DST_COLOR, GL_ZERO);
    glEnable(GL_BLEND);
    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_ITEMS]);
    glTexCoordPointer(2,GL_FLOAT,0,panel_mask_texcoord);
    glVertexPointer(3,GL_FLOAT,0,panel_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glBlendFunc(GL_ONE, GL_ONE);
    glEnable(GL_BLEND);
    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_ITEMS]);
    glTexCoordPointer(2,GL_FLOAT,0,panel_texcoord);
    glVertexPointer(3,GL_FLOAT,0,panel_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glPopMatrix();
    
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnableClientState(GL_NORMAL_ARRAY);
}

/********************************************************************
** Draw the calculated score.
*********************************************************************/
void DrawScore(void)
{	
	GLfloat Score_tex[6][2] = {0};
	int count;

    glDisable(GL_DEPTH_TEST);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisable(GL_LIGHTING);
	
	for(count=0;count<12;count++)
	{
		Score_tex[0][0] = 0.062 * ScoreNum[count] + 0.062;
		Score_tex[1][0] = 0.062 * ScoreNum[count];
		Score_tex[2][0] = 0.062 * ScoreNum[count];
		Score_tex[3][0] = 0.062 * ScoreNum[count];
		Score_tex[4][0] = 0.062 * ScoreNum[count] + 0.062;
		Score_tex[5][0] = 0.062 * ScoreNum[count] + 0.062;
		
		Score_tex[0][1] = 0.062;
		Score_tex[1][1] = 0.062;
		Score_tex[2][1] = 0.124;
		Score_tex[3][1] = 0.124;
		Score_tex[4][1] = 0.124;
		Score_tex[5][1] = 0.062;

		glPushMatrix(); 
		if (GameState == GSTATE_TITLE)
		{
			glTranslatef(-1.0f + ( count * 0.10 ) , 0.00f ,DEF_DEPTH_5);
			glTranslatef(0.425f,0.95f,0.0f);
			glScalef(0.05,0.04,0.1);
		}
		else
		{
			glTranslatef(-1.0f + ( count * 0.10 ) , 0.00f ,DEF_DEPTH_5);
			//glTranslatef(0.18f,0.87f,0.0f);
			glTranslatef(0.425f,0.87f,0.0f);
			glScalef(0.1,0.05,0.1);
		}

	    glBlendFunc(GL_ONE, GL_ONE);
	    glEnable(GL_BLEND);
		glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_FONT]);
		glVertexPointer(3,GL_FLOAT, 0, Score_arrays);
		glTexCoordPointer(2, GL_FLOAT, 0, Score_tex);
		glDrawArrays(GL_TRIANGLES,0,6);

		glPopMatrix();
	}

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnableClientState(GL_NORMAL_ARRAY);
}

/********************************************************************
** Draw loading screen.
*********************************************************************/
void DrawLoading(void)
{
    glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

    glPushMatrix();

    glDisable(GL_BLEND);
    
    glTranslatef(0,0,DEF_DEPTH_5);

    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_LOADING]);
    glTexCoordPointer(2,GL_FLOAT,0,plane_texcoord);
    glNormalPointer(GL_FLOAT,0,plane_normal);
    glVertexPointer(3,GL_FLOAT,0,plane_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glPopMatrix();    

    glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
}

/********************************************************************
** Draw Title Screen
*********************************************************************/
void DrawTitleScreen(void)
{
    glDisable(GL_DEPTH_TEST);

    glPushMatrix();

    glDisable(GL_BLEND);
    
    glTranslatef(0,0,DEF_DEPTH_0);

    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_TSCREEN]);
    glTexCoordPointer(2,GL_FLOAT,0,plane_texcoord);
    glNormalPointer(GL_FLOAT,0,plane_normal);
    glVertexPointer(3,GL_FLOAT,0,plane_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glPopMatrix();    

    glEnable(GL_DEPTH_TEST);

}

/********************************************************************
** Draw Title Logo
*********************************************************************/
void DrawTitleLogo(void)
{
	static
	GLfloat	titleX 		= 0.0f;
	GLfloat	to_titleX 	= 1.0f;
	static
	char	Blinking	= 0;
	
    glDisable(GL_DEPTH_TEST);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisable(GL_LIGHTING);

    glPushMatrix();

	if (titleX < to_titleX) 
	{
		titleX += 0.05f;
	}
	else
	{
		Blinking ++;
		if (Blinking > 100) Blinking = 0;
	}
	
    glTranslatef(0.0f,0.4f,DEF_DEPTH_1);
    glScalef(1.0f,0.6f,1.0f);
    glScalef(titleX,titleX,1.0f);

	if (Blinking < 80)
	{	
	    glBlendFunc(GL_DST_COLOR, GL_ZERO);
	    glEnable(GL_BLEND);
	    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_TITLE]);
	    glTexCoordPointer(2,GL_FLOAT,0,half_down_plane_texcoord);
	    glVertexPointer(3,GL_FLOAT,0,half_plane_vertex);
	    glDrawArrays(GL_TRIANGLES,0,6);

	    glBlendFunc(GL_ONE, GL_ONE);
	    glEnable(GL_BLEND);
	    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_TITLE]);
	    glTexCoordPointer(2,GL_FLOAT,0,half_up_plane_texcoord);
	    glVertexPointer(3,GL_FLOAT,0,half_plane_vertex);
	    glDrawArrays(GL_TRIANGLES,0,6);
	}
	
    glPopMatrix();
    
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnableClientState(GL_NORMAL_ARRAY);
}

/********************************************************************
** Proceeds player movement.
*********************************************************************/
void PlayerCheck(void)
{
	if (!crushed)
	{
		// -- Do Move --
		// Both key can be checked !
		if (player.moveUP) 
		{
			player.coord_Y += 0.02f;
			if (player.angle_Y < 0.0f) player.angle_Y = 0.0f;
			player.angle_Y += 3.0f;
		}
		else
		if (player.moveDOWN)
		{
			player.coord_Y -= 0.02f;
			if (player.angle_Y > 0.0f) player.angle_Y = 0.0f;
			player.angle_Y -= 3.0f;
		}

		if (player.moveLEFT) 
		{
			player.coord_X -= 0.02f;
			if (player.angle_X > 0.0f) player.angle_X = 0.0f;
			player.angle_X -= 3.0f;
		}
		else
		if (player.moveRIGHT) 
		{
			player.coord_X += 0.02f;
			if (player.angle_X < 0.0f) player.angle_X = 0.0f;
			player.angle_X += 3.0f;
		}
		
		// -- Movement limit --
		if (player.coord_X < -0.95f) player.coord_X = -0.95f;
		if (player.coord_X >  0.95f) player.coord_X =  0.95f;
		if (player.coord_Y < -0.95f) player.coord_Y = -0.95f;
		if (player.coord_Y >  0.40f) player.coord_Y =  0.40f;
		
		// -- Angle limit
		if (player.angle_X < -30.0f) player.angle_X = -30.0f;
		if (player.angle_X >  30.0f) player.angle_X =  30.0f;
		if (player.angle_Y < -30.0f) player.angle_Y = -30.0f;
		if (player.angle_Y >  30.0f) player.angle_Y =  30.0f;
		
		// -- Move back origin angle.
		if ((!player.moveUP) && (!player.moveDOWN))
		{
			if (player.angle_Y > 0.0f) player.angle_Y -= 2.0f;
			if (player.angle_Y < 0.0f) player.angle_Y += 2.0f;
		}
		if ((!player.moveLEFT) && (!player.moveRIGHT))
		{
			if (player.angle_X > 0.0f) player.angle_X -= 2.0f;
			if (player.angle_X < 0.0f) player.angle_X += 2.0f;
		}
	}
}

/********************************************************************
** Draws HOW-TO-PLAY?
*********************************************************************/
void DrawHowToPlay(void)
{
	glDisable(GL_DEPTH_TEST);

    glPushMatrix();

    glDisable(GL_BLEND);
    
    glTranslatef(0,0,DEF_DEPTH_0);

    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_HOW2PLAY]);
    glTexCoordPointer(2,GL_FLOAT,0,plane_texcoord);
    glNormalPointer(GL_FLOAT,0,plane_normal);
    glVertexPointer(3,GL_FLOAT,0,plane_vertex);
    glDrawArrays(GL_TRIANGLES,0,6);

    glPopMatrix();    

    glEnable(GL_DEPTH_TEST);
}

/********************************************************************
** Draw Title Menu
*********************************************************************/
void DrawTitleMenu(void)
{
	char	count;
	static
	GLfloat	dynamics = 0.0f;

	dynamics  = rand()%2;
	dynamics -= 1.0f;
	dynamics /= 100.0f;

    glDisable(GL_DEPTH_TEST);
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisable(GL_LIGHTING);

    // -- Draw back bar
    glPushMatrix();

    glTranslatef(0,-0.01,DEF_DEPTH_3);
    glRotatef(-180,1,0,0);

    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glEnable(GL_BLEND);
    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_TEXTS]);
    glTexCoordPointer(2,GL_FLOAT,0,menu_selectbar_texcoord);
    glVertexPointer(3,GL_FLOAT,0,&menu_items_vertex[menu_item_select * (6*3)]);
    glDrawArrays(GL_TRIANGLES,0,6);
	
    glPopMatrix();

	// -- Draw menu
    glPushMatrix();

    glTranslatef(0,0,DEF_DEPTH_4);
    glRotatef(-180,1,0,0);

	for(count=0;count<5;count++)
	{			
	    glBlendFunc(GL_ONE, GL_ONE);
	    glEnable(GL_BLEND);
	    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_TEXTS]);
	    glTexCoordPointer(2,GL_FLOAT,0,menu_items_texcoords);
	    glVertexPointer(3,GL_FLOAT,0,menu_items_vertex);
	    glDrawArrays(GL_TRIANGLES,count * 6,6);
	}

	for(count=10;count<13;count++)
	{	
	    glBlendFunc(GL_ONE, GL_ONE);
	    glEnable(GL_BLEND);
	    glTranslatef(0,dynamics,0);

	    glBindTexture(GL_TEXTURE_2D,TextureID[DA_TEX_TEXTS]);
	    glTexCoordPointer(2,GL_FLOAT,0,menu_items_texcoords);
	    glVertexPointer(3,GL_FLOAT,0,menu_items_vertex);
	    glDrawArrays(GL_TRIANGLES,count * 6,6);
	}
	
    glPopMatrix();
        
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glEnableClientState(GL_NORMAL_ARRAY);
}
	
/********************************************************************
** Loads textures from file system.
*********************************************************************/
void TextureLoading(void)
{
    // Binding Textures from File System 
    BindingTexture(LoadTextureFromFS(DEF_FILE_FLIGHT)	,DA_TEX_FLIGHT,128,128);
    BindingTexture(LoadTextureFromFS(DEF_FILE_FONT)		,DA_TEX_FONT,256,256);
    BindingTexture(LoadTextureFromFS(DEF_FILE_ITEMS)	,DA_TEX_ITEMS,256,256);
    BindingTexture(LoadTextureFromFS(DEF_FILE_SAPCE)	,DA_TEX_SPACE,128,128);
    BindingTexture(LoadTextureFromFS(DEF_FILE_TEXTS)	,DA_TEX_TEXTS,256,256);
    BindingTexture(LoadTextureFromFS(DEF_FILE_TITLE)	,DA_TEX_TITLE,128,128);
    BindingTexture(LoadTextureFromFS(DEF_FILE_TSCREEN)	,DA_TEX_TSCREEN,256,256);
    BindingTexture(LoadTextureFromFS(DEF_FILE_HOW2PLAY)	,DA_TEX_HOW2PLAY,256,256);

	LoadFromFS = 1;	
}

/********************************************************************
** Set some function and parameters to start the game
*********************************************************************/
void StartSetup(void)
{
	char count;
	
	memset(&player,0,sizeof(player));
	
	for(count=0;count<DEF_MAX_CREATE_BULLETS;count++)
	{
		ResetBullets(count);
	}
	
	crushcount = 0;	
	game_score = 0;
	hit_level  = 0;
	
	start_time = GetTickCount();
}

/********************************************************************
** Initialize 
*********************************************************************/
void InitGL(void)
{    
    void* pTexture = NULL;
	
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
 	setPersfective(0,0.8,0,500);
 
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

	SetArrays();
  	
    glShadeModel(GL_SMOOTH);
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_col);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
	glLightfv(GL_LIGHT0, GL_SPECULAR, light_spc);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);

    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glMaterialfv(GL_FRONT_AND_BACK , GL_AMBIENT_AND_DIFFUSE , materialCol); 

    glClearColor(0,0,0,1);
    
    //==============================================================

    glGenTextures(DA_TEX_MAX-1,TextureID);
    
    // First time , shows " LOADING " image ...
    BindingTexture(texture_loading        ,DA_TEX_LOADING,128,128);

    glEnable(GL_TEXTURE_2D);
	
	MakeMenuCoordinates(); 
	SetArrays();
}

/********************************************************************
** Update current situation
*********************************************************************/
void UpdateGL(void)
{

#ifdef _WIN32

    // This code for anti-over-speed-rendering.
	// keep frame rate for Win32 fastest system.
	// ex ) Pentium4 2.4GHz + ATi Radeon9600pro = renders about 160fps.

    static unsigned int preTick,nowTick = 0;
    unsigned int diffTick;
    unsigned int checkTick = DEF_FPS_LIMIT; // <- frame per second limit (set as 100 fps)
    
    if (preTick == 0)
    {
        nowTick = GetTickCount();
        preTick = nowTick - checkTick;
    }
    else    nowTick = GetTickCount();
    
    diffTick = nowTick - preTick;
    
    if (diffTick < checkTick) return;
#endif

	if (ShowedLoading)
	{
		if (!LoadFromFS)
		{
			TextureLoading();
			LoadFromFS = 1;
			GameState  = GSTATE_TITLE;
		}
	}

	// -- time check
	passed_time = GetTickCount() - start_time;
	
	// -- Craft movement --
	PlayerCheck();
	
	// -- Game state process -- 
	switch(GameState)
	{
		case GSTATE_TITLE:
			crushed = 0;
			ResetAllBullets();

			if (MusicState != GSTATE_TITLE)
			{
				StopPlay(0);
				PlayFile(0,DEF_FILE_MUSIC1,1);
				MusicState = GameState;
			}
			break;
		case GSTATE_LEVEL:
		case GSTATE_OPTION:
			break;
			
		case GSTATE_GAME_START:
			if (MusicState != GSTATE_GAME_START)
			{
				StopPlay(0);
				PlayFile(0,DEF_FILE_MUSIC2,1);

				StopPlay(1);
				PlayFile(1,DEF_FILE_MSTART,0);			
				GameState = GSTATE_GAME_PLAYING;
				MusicState = GameState;
			}
			break;
			
		case GSTATE_GAME_PLAYING:
			MoveBullets();
			CalScore();
			break;
			
		case GSTATE_GAME_DIED:
			if (MusicState != GSTATE_GAME_DIED)
			{
				StopPlay(0);
				PlayFile(0,DEF_FILE_MUSIC3,0);
				MusicState = GameState;
			}

			if (crushcount > 200) 
			{
				GameState = GSTATE_TITLE;
				crushcount = 0;
			}
			break;
			
		case GSTATE_GAME_END:
			break;
	}
	
#ifdef _WIN32
	// this is code for anti-over-speed-rendering.	
    preTick = nowTick;
#endif

}

/********************************************************************
** Draws models .
*********************************************************************/
void DrawGL(void)
{
	int count = 0;
	
#ifdef _WIN32

    // This code for anti-over-speed-rendering.
	// keep frame rate for Win32 fastest system.
	// ex ) Pentium4 2.4GHz + ATi Radeon9600pro = renders about 160fps.

    static unsigned int preTick,nowTick = 0;
    unsigned int diffTick;
    unsigned int checkTick = DEF_FPS_LIMIT; // <- frame per second limit (set as 100 fps)
    
    if (preTick == 0)
    {
        nowTick = GetTickCount();
        preTick = nowTick - checkTick;
    }
    else    nowTick = GetTickCount();
    
    diffTick = nowTick - preTick;
    
    if (diffTick < checkTick) return;
#endif

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Before loads texture from filesystem , draws loading image , first
	if (!ShowedLoading)
	{
		DrawLoading();
		ShowedLoading = 1;
	}
	else
	{
		switch(GameState)
		{
			case GSTATE_TITLE :
				DrawTitleScreen();
				DrawTitleLogo();
				DrawTitleMenu();
				DrawScore();
				break;
			
			case GSTATE_HOW2PLAY:
				DrawHowToPlay();
				break;
			
			case GSTATE_GAME_START:
			case GSTATE_GAME_PLAYING:
			case GSTATE_GAME_DIED:
			case GSTATE_GAME_END:
				DrawSpace();
				DrawCraft();
				DrawBullets();
				DrawScorePanel();
				
				if ((GameState == GSTATE_GAME_DIED) || (GameState == GSTATE_GAME_END))
				{
					DrawGameOver();
					if (crushcount < 100)
					{
					DrawScore();
					}
				}
				else 
				{
					DrawScore();
				}
				
				DrawBlow();
				
				break;
				
			default:
				break;
		}
	}

#ifdef _WIN32

	// this is code for anti-over-speed-rendering.
	
    preTick = nowTick;
#endif

}

void KeyControl(int key)
{
	switch(GameState)
	{
		case GSTATE_TITLE :
			StopPlay(3);
			PlayFile(3,DEF_FILE_MBUTTON,0);
			switch(key)
			{
				case KEY_UP	:
					menu_item_select--;
					if (menu_item_select < 0) menu_item_select = 4;
					break;
				case KEY_DOWN :
					menu_item_select++;
					if (menu_item_select > 4) menu_item_select = 0;
					break;
				case KEY_FIRE:
				case KEY_SELECT :
					StopPlay(3);
					PlayFile(3,DEF_FILE_MMOVE,0);
					switch(menu_item_select)
					{
						case 0:
							StartSetup();
							GameState = GSTATE_GAME_START;
							break;
						case 3: // HowToPlay ?
							GameState = GSTATE_HOW2PLAY;
							break;
						case 4: // Exit Game !
							CloseGL();
							Sleep(1000);
							exit(0);
							break;
					}
					break;
			}
			break;
			
		case GSTATE_HOW2PLAY:
			StopPlay(3);
			PlayFile(3,DEF_FILE_MMOVE,0);
			GameState = GSTATE_TITLE;
			break;

		case GSTATE_GAME_START:
			GameState = GSTATE_GAME_PLAYING;
			break;
			
		case GSTATE_GAME_PLAYING :
			switch(key)
			{
				case KEY_UP:
					player.moveUP   = 1;
					player.moveDOWN = 0;
					break;
				
				case KEY_DOWN:
					player.moveUP   = 0;
					player.moveDOWN = 1;
					break;
				
				case KEY_LEFT:
					player.moveLEFT  = 1;
					player.moveRIGHT = 0;
					break;
				
				case KEY_RIGHT:
					player.moveLEFT  = 0;
					player.moveRIGHT = 1;
					break;
				
				case KEY_FIRE :
					break;
					
				case KEY_CANCEL :
					hit_level = 4;
					DoCrush();
					break;
			}
			break;
		
		case GSTATE_GAME_DIED :
		case GSTATE_GAME_END:
			if (crushcount > 100)
			{
				ResetAllBullets();
				GameState = GSTATE_TITLE;
			}
			break;
	}
}

/********************************************************************
** Key up event.

*********************************************************************/
void KeyUpControl(int key)
{
	switch(GameState)
	{
		case GSTATE_GAME_PLAYING :
			switch(key)
			{
				case KEY_UP:
					player.moveUP   = 0;
					break;
				
				case KEY_DOWN:
					player.moveDOWN = 0;
					break;
				
				case KEY_LEFT:
					player.moveLEFT  = 0;
					break;
				
				case KEY_RIGHT:
					player.moveRIGHT = 0;
					break;
			}
			break;
	}
}

/********************************************************************
** It be called when states going out.
*********************************************************************/
void CloseGL(void)
{
    int Count = 0;
    
	// If you have been used glGenTextures()
	// You must use glDeleteTextures() when finished using Textures.
	glDeleteTextures(DA_TEX_MAX-1,TextureID);
	
	for(Count=0;Count<DA_TEX_MAX;Count++)
	{
		if (Textures[Count])
		{
			free(Textures[Count]);
		}
	}
}
